@@ -1,6 +1,29 @@
-/* ************************************************************************
- * Copyright (c) 2022 Advanced Micro Devices, Inc.
- * ************************************************************************ */
+/* **************************************************************************
+ * Copyright (C) 2022-2023 Advanced Micro Devices, Inc. All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
+ *
+ * 1. Redistributions of source code must retain the above copyright
+ *    notice, this list of conditions and the following disclaimer.
+ *
+ * 2. Redistributions in binary form must reproduce the above copyright
+ *    notice, this list of conditions and the following disclaimer in the
+ *    documentation and/or other materials provided with the distribution.
+ *
+ * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
+ * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+ * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+ * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
+ * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
+ * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
+ * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
+ * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
+ * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
+ * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
+ * SUCH DAMAGE.
+ * *************************************************************************/
 
 #include <cstdlib>
 #if __has_include(<filesystem>)
@@ -19,8 +42,8 @@ namespace fs = std::experimental::filesystem;
 #include <rocblas/rocblas.h>
 #include <rocsolver/rocsolver.h>
 
-#include "client_environment_helpers.hpp"
-#include "clientcommon.hpp"
+#include "common/misc/client_environment_helpers.hpp"
+#include "common/misc/clientcommon.hpp"
 
 using ::testing::Matcher;
 using ::testing::MatchesRegex;
@@ -49,7 +72,7 @@ protected:
         {
             if(HasFailure() && std::getenv("ROCSOLVER_TEST_DEBUG"))
                 fmt::print(stderr, "ROCSOLVER_TEST_DEBUG is set so {} was not removed.\n",
-                           log_filepath);
+                           log_filepath.string());
             else
                 EXPECT_TRUE(fs::remove(log_filepath));
         }
@@ -129,7 +152,7 @@ TEST_F(checkin_misc_LOGGING, rocblas_layer_mode_log_trace)
         "rocSOLVER Version: .*",
         "rocBLAS Version: .*",
         ".*ENTER rocsolver_dgetrf_strided_batched trace tree.*",
-        ".*getrf.*m: 25, n: 25, shiftA: 0, lda: 25, shiftP: 0, bc: 3.*",
+        ".*getrf.*m: 25, n: 25, shiftA: 0, inca: 1, lda: 25, shiftP: 0, bc: 3.*",
         ".*EXIT rocsolver_dgetrf_strided_batched trace tree.*",
         "\\s*",
     };
@@ -285,7 +308,7 @@ TEST_F(checkin_misc_LOGGING, rocsolver_log_restore_defaults_resets_max_levels)
         "rocSOLVER Version: .*",
         "rocBLAS Version: .*",
         ".*ENTER rocsolver_dgetrf_strided_batched trace tree.*",
-        ".*getrf.*m: 25, n: 25, shiftA: 0, lda: 25, shiftP: 0, bc: 3.*",
+        ".*getrf.*m: 25, n: 25, shiftA: 0, inca: 1, lda: 25, shiftP: 0, bc: 3.*",
         ".*EXIT rocsolver_dgetrf_strided_batched trace tree.*",
         "\\s*",
     };
@@ -309,8 +332,8 @@ TEST_F(checkin_misc_LOGGING, rocblas_layer_mode_log_trace_tree)
         "rocSOLVER Version: .*",
         "rocBLAS Version: .*",
         ".*ENTER rocsolver_dgetrf_strided_batched trace tree.*",
-        ".*getrf.*m: 25, n: 25, shiftA: 0, lda: 25, shiftP: 0, bc: 3.*",
-        "    .*getf2.*m: 25, n: 25, shiftA: 0, lda: 25, shiftP: 0, bc: 3.*",
+        ".*getrf.*m: 25, n: 25, shiftA: 0, inca: 1, lda: 25, shiftP: 0, bc: 3.*",
+        "    .*getf2.*m: 25, n: 25, shiftA: 0, inca: 1, lda: 25, shiftP: 0, bc: 3.*",
         ".*EXIT rocsolver_dgetrf_strided_batched trace tree.*",
         "\\s*",
     };
