@@ -1,6 +1,29 @@
-/* ************************************************************************
- * Copyright (c) 2019-2022 Advanced Micro Devices, Inc.
- * ************************************************************************ */
+/* **************************************************************************
+ * Copyright (C) 2019-2025 Advanced Micro Devices, Inc. All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
+ *
+ * 1. Redistributions of source code must retain the above copyright
+ *    notice, this list of conditions and the following disclaimer.
+ *
+ * 2. Redistributions in binary form must reproduce the above copyright
+ *    notice, this list of conditions and the following disclaimer in the
+ *    documentation and/or other materials provided with the distribution.
+ *
+ * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
+ * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+ * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+ * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
+ * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
+ * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
+ * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
+ * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
+ * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
+ * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
+ * SUCH DAMAGE.
+ * *************************************************************************/
 
 #pragma once
 
@@ -10,6 +33,8 @@
 #include "lib_macros.hpp"
 #include "libcommon.hpp"
 
+ROCSOLVER_BEGIN_NAMESPACE
+
 /*
  * ===========================================================================
  *    common location for device functions and kernels that are used across
@@ -18,9 +43,6 @@
  * ===========================================================================
  */
 
-#define BS1 256 // generic 1 dimensional thread-block size used to call common kernels
-#define BS2 32 // generic 2 dimensional thread-block size used to call common kernels
-
 // **********************************************************
 // device functions that are used by many kernels
 // **********************************************************
@@ -53,30 +75,61 @@ __device__ void swap(const rocblas_int n, T* a, const rocblas_int inca, T* b, co
         swap(a[inca * tid], b[incb * tid]);
 }
 
+template <typename T, std::enable_if_t<!rocblas_is_complex<T>, int> = 0>
+__device__ __inline__ T shift_left(T& value, int lane_delta)
+{
+    T r = value;
+    r = __shfl_down(r, lane_delta);
+    return r;
+}
+
+template <typename T, std::enable_if_t<rocblas_is_complex<T>, int> = 0>
+__device__ __inline__ T shift_left(T& value, int lane_delta)
+{
+    using S = decltype(std::real(T{}));
+    S r = value.real();
+    S i = value.imag();
+    r = __shfl_down(r, lane_delta);
+    i = __shfl_down(i, lane_delta);
+    return rocblas_complex_num<S>(r, i);
+}
+
 /** SWAPVECT device function swap vectors a and b of dimension n **/
 template <typename T>
 __device__ void
     swapvect(const rocblas_int n, T* a, const rocblas_int inca, T* b, const rocblas_int incb)
 {
-    for(rocblas_int i = 0; i < n; ++i)
-        swap(a[inca * i], b[incb * i]);
+    // check common special case
+    if((inca == 1) && (incb == 1))
+    {
+        for(rocblas_int i = 0; i < n; ++i)
+        {
+            swap(a[i], b[i]);
+        };
+    }
+    else
+    {
+        for(rocblas_int i = 0; i < n; ++i)
+            swap(a[inca * i], b[incb * i]);
+    };
 }
 
 /** FIND_MAX_TRIDIAG finds the element with the largest magnitude in the
     tridiagonal matrix **/
 template <typename T>
-__device__ T find_max_tridiag(const rocblas_int start, const rocblas_int end, T* D, T* E)
+__device__ __host__ T find_max_tridiag(const rocblas_int start, const rocblas_int end, T* D, T* E)
 {
     T anorm = abs(D[end]);
     for(int i = start; i < end; i++)
-        anorm = max(anorm, max(abs(D[i]), abs(E[i])));
+        anorm = std::fmax(anorm, std::fmax(abs(D[i]), abs(E[i])));
     return anorm;
 }
 
 /** SCALE_TRIDIAG scales the elements of the tridiagonal matrix by a given
     scale factor **/
 template <typename T>
-__device__ void scale_tridiag(const rocblas_int start, const rocblas_int end, T* D, T* E, T scale)
+__device__ __host__ void
+    scale_tridiag(const rocblas_int start, const rocblas_int end, T* D, T* E, T scale)
 {
     D[end] *= scale;
     for(int i = start; i < end; i++)
@@ -86,6 +139,457 @@ __device__ void scale_tridiag(const rocblas_int start, const rocblas_int end, T*
     }
 }
 
+template <typename S, typename I>
+__device__ static void shell_sort_ascending(const I n, S* a, I* map = nullptr)
+{
+    // -----------------------------------------------
+    // Sort array a[0...(n-1)] and generate permutation vector
+    // in map[] if map[] is available
+    // Note: performs in a single thread block
+    // -----------------------------------------------
+    if((n <= 0) || (a == nullptr))
+    {
+        return;
+    };
+
+    bool const is_root_thread
+        = (hipThreadIdx_x == 0) && (hipThreadIdx_y == 0) && (hipThreadIdx_z == 0);
+
+    int constexpr ngaps = 8;
+    int const gaps[ngaps] = {701, 301, 132, 57, 23, 10, 4, 1}; // # Ciura gap sequence
+
+    auto const k_start = hipThreadIdx_x + hipThreadIdx_y * hipBlockDim_x
+        + hipThreadIdx_z * (hipBlockDim_x * hipBlockDim_y);
+
+    auto const k_inc = (hipBlockDim_x * hipBlockDim_y) * hipBlockDim_z;
+
+    bool const has_map = (map != nullptr);
+
+    __syncthreads();
+    if(has_map)
+    {
+        for(auto k = k_start; k < n; k += k_inc)
+        {
+            map[k] = k;
+        };
+    };
+    __syncthreads();
+
+    if(is_root_thread)
+    {
+        for(auto igap = 0; igap < ngaps; igap++)
+        {
+            auto const gap = gaps[igap];
+
+            for(I i = gap; i < n; i += 1)
+            {
+                {
+                    S const temp = a[i];
+                    auto const itemp = (has_map) ? map[i] : 0;
+
+                    auto j = i;
+
+                    while((j >= gap) && (a[j - gap] > temp))
+                    {
+                        a[j] = a[j - gap];
+                        if(has_map)
+                        {
+                            map[j] = map[j - gap];
+                        };
+                        j -= gap;
+                    };
+
+                    a[j] = temp;
+                    if(has_map)
+                    {
+                        map[j] = itemp;
+                    };
+                };
+            };
+        };
+    };
+    __syncthreads();
+#ifdef NDEBUG
+#else
+    if(n >= 2)
+    {
+        for(auto k = k_start; k < (n - 1); k += k_inc)
+        {
+            assert(a[k] <= a[k + 1]);
+        };
+    };
+    __syncthreads();
+#endif
+}
+
+template <typename S, typename I>
+__device__ static void shell_sort_descending(const I n, S* a, I* map = nullptr)
+{
+    // -----------------------------------------------
+    // Sort array a[0...(n-1)] and generate permutation vector
+    // in map[] if map[] is available
+    // Note: performs in a single thread block
+    // -----------------------------------------------
+
+    if((n <= 0) || (a == nullptr))
+    {
+        return;
+    };
+
+    bool const is_root_thread
+        = (hipThreadIdx_x == 0) && (hipThreadIdx_y == 0) && (hipThreadIdx_z == 0);
+
+    int constexpr ngaps = 8;
+    int const gaps[ngaps] = {701, 301, 132, 57, 23, 10, 4, 1}; // # Ciura gap sequence
+
+    auto const k_start = hipThreadIdx_x + hipThreadIdx_y * hipBlockDim_x
+        + hipThreadIdx_z * (hipBlockDim_x * hipBlockDim_y);
+
+    auto const k_inc = (hipBlockDim_x * hipBlockDim_y) * hipBlockDim_z;
+
+    bool const has_map = (map != nullptr);
+
+    __syncthreads();
+    if(has_map)
+    {
+        for(auto k = k_start; k < n; k += k_inc)
+        {
+            map[k] = k;
+        };
+    };
+    __syncthreads();
+
+    if(is_root_thread)
+    {
+        for(auto igap = 0; igap < ngaps; igap++)
+        {
+            auto const gap = gaps[igap];
+
+            for(rocblas_int i = gap; i < n; i += 1)
+            {
+                {
+                    S const temp = a[i];
+                    auto const itemp = (has_map) ? map[i] : 0;
+
+                    auto j = i;
+
+                    while((j >= gap) && (a[j - gap] < temp))
+                    {
+                        a[j] = a[j - gap];
+                        if(has_map)
+                        {
+                            map[j] = map[j - gap];
+                        };
+                        j -= gap;
+                    };
+
+                    a[j] = temp;
+                    if(has_map)
+                    {
+                        map[j] = itemp;
+                    };
+                };
+            };
+        };
+    };
+    __syncthreads();
+#ifdef NDEBUG
+#else
+    if(n >= 2)
+    {
+        for(auto k = k_start; k < (n - 1); k += k_inc)
+        {
+            assert(a[k] >= a[k + 1]);
+        };
+    };
+    __syncthreads();
+#endif
+}
+
+template <typename S, typename I>
+__device__ static void shell_sort(const I n, S* a, I* map = nullptr, const bool is_ascending = true)
+{
+    if((n <= 0) || (a == nullptr))
+    {
+        return;
+    };
+
+    if(is_ascending)
+    {
+        shell_sort_ascending(n, a, map);
+    }
+    else
+    {
+        shell_sort_descending(n, a, map);
+    };
+}
+
+template <typename S, typename I>
+__device__ static void selection_sort_ascending(const I n, S* D, I* map = nullptr)
+{
+    // ---------------------------------------------------
+    // Sort entries in D[0...(n-1)]
+    // generates permutation vector in map[] if map[] is available
+    // Note: performs in a single thread block
+    // ---------------------------------------------------
+    if((n <= 0) || (D == nullptr))
+    {
+        return;
+    };
+
+    auto const tid = hipThreadIdx_x + hipThreadIdx_y * hipBlockDim_x
+        + hipThreadIdx_z * (hipBlockDim_x * hipBlockDim_y);
+
+    auto const nthreads = (hipBlockDim_x * hipBlockDim_y) * hipBlockDim_z;
+
+    auto const k_start = tid;
+    auto const k_inc = nthreads;
+    bool const is_root_thread = (tid == 0);
+
+    bool const has_map = (map != nullptr);
+
+    __syncthreads();
+
+    if(has_map)
+    {
+        for(auto k = k_start; k < n; k += k_inc)
+        {
+            map[k] = k;
+        };
+    };
+
+    __syncthreads();
+
+    if(is_root_thread)
+    {
+        for(I ii = 1; ii < n; ii++)
+        {
+            auto l = ii - 1;
+            auto m = l;
+            auto p = D[l];
+            auto ip = (has_map) ? map[l] : 0;
+
+            for(auto j = ii; j < n; j++)
+            {
+                if(D[j] < p)
+                {
+                    m = j;
+                    p = D[j];
+                    if(has_map)
+                    {
+                        ip = map[j];
+                    };
+                }
+            }
+            if(m != l)
+            {
+                D[m] = D[l];
+                D[l] = p;
+
+                if(has_map)
+                {
+                    map[m] = map[l];
+                    map[l] = ip;
+                };
+            }
+        }
+    }
+    __syncthreads();
+#ifdef NDEBUG
+#else
+    if(n >= 2)
+    {
+        for(auto k = k_start; k < (n - 1); k += k_inc)
+        {
+            assert(D[k] <= D[k + 1]);
+        };
+    };
+    __syncthreads();
+#endif
+}
+
+template <typename S, typename I>
+__device__ static void selection_sort_descending(const I n, S* D, I* map = nullptr)
+{
+    // ---------------------------------------------------
+    // Sort entries in D[0...(n-1)]
+    // generates permutation vector in map[] if map[] is available
+    // Note: performs in a single thread block
+    // ---------------------------------------------------
+    if((n <= 0) || (D == nullptr))
+    {
+        return;
+    };
+
+    auto const tid = hipThreadIdx_x + hipThreadIdx_y * hipBlockDim_x
+        + hipThreadIdx_z * (hipBlockDim_x * hipBlockDim_y);
+
+    auto const nthreads = (hipBlockDim_x * hipBlockDim_y) * hipBlockDim_z;
+
+    auto const k_start = tid;
+    auto const k_inc = nthreads;
+    bool const is_root_thread = (tid == 0);
+
+    bool const has_map = (map != nullptr);
+
+    __syncthreads();
+
+    if(has_map)
+    {
+        for(auto k = k_start; k < n; k += k_inc)
+        {
+            map[k] = k;
+        };
+    };
+
+    __syncthreads();
+
+    if(is_root_thread)
+    {
+        for(I ii = 1; ii < n; ii++)
+        {
+            auto l = ii - 1;
+            auto m = l;
+            auto p = D[l];
+            auto ip = (has_map) ? map[l] : 0;
+
+            for(auto j = ii; j < n; j++)
+            {
+                if(D[j] > p)
+                {
+                    m = j;
+                    p = D[j];
+                    if(has_map)
+                    {
+                        ip = map[j];
+                    };
+                }
+            }
+            if(m != l)
+            {
+                D[m] = D[l];
+                D[l] = p;
+
+                if(has_map)
+                {
+                    map[m] = map[l];
+                    map[l] = ip;
+                };
+            }
+        }
+    }
+    __syncthreads();
+#ifdef NDEBUG
+#else
+    if(n >= 2)
+    {
+        for(auto k = k_start; k < (n - 1); k += k_inc)
+        {
+            assert(D[k] >= D[k + 1]);
+        };
+    };
+    __syncthreads();
+#endif
+}
+
+template <typename S, typename I>
+__device__ static void selection_sort(const I n, S* a, I* map = nullptr, const bool is_ascending = true)
+{
+    if((n <= 0) || (a == nullptr))
+    {
+        return;
+    };
+
+    if(is_ascending)
+    {
+        selection_sort_ascending(n, a, map);
+    }
+    else
+    {
+        selection_sort_descending(n, a, map);
+    };
+}
+
+template <typename T, typename I>
+__device__ static void permute_swap(const I n, T* C, I ldc, I* map, const I nev = -1)
+{
+    // --------------------------------------------
+    // perform swaps to implement permutation vector
+    // the permutation vector will be restored to the
+    // identity permutation 0,1,2,...
+    //
+    // Note: performs in a thread block
+    // --------------------------------------------
+    {
+        bool const has_work = (n >= 1) && (C != nullptr) && (ldc >= 1) && (map != nullptr);
+        if(!has_work)
+        {
+            return;
+        }
+    }
+
+    auto const tid = hipThreadIdx_x + hipThreadIdx_y * hipBlockDim_x
+        + hipThreadIdx_z * (hipBlockDim_x * hipBlockDim_y);
+
+    auto const nthreads = (hipBlockDim_x * hipBlockDim_y) * hipBlockDim_z;
+
+    auto const k_start = tid;
+    auto const k_inc = nthreads;
+
+    bool const is_root_thread = (tid == 0);
+
+    auto const nn = (nev >= 0) ? nev : n;
+
+    for(I i = 0; i < nn; i++)
+    {
+        __syncthreads();
+
+        while(map[i] != i)
+        {
+            auto const map_i = map[i];
+            auto const map_ii = map[map[i]];
+
+            __syncthreads();
+
+            if(is_root_thread)
+            {
+                map[map_i] = map_i;
+                map[i] = map_ii;
+            }
+
+            __syncthreads();
+
+            // ---------------------------------------
+            // swap columns C( 0:(n-1),map_i) and C( 0:(n-1),map_ii)
+            // ---------------------------------------
+
+            for(auto k = k_start; k < n; k += k_inc)
+            {
+                auto const k_map_i = k + (map_i * static_cast<int64_t>(ldc));
+                auto const k_map_ii = k + (map_ii * static_cast<int64_t>(ldc));
+
+                auto const ctemp = C[k_map_i];
+                C[k_map_i] = C[k_map_ii];
+                C[k_map_ii] = ctemp;
+            }
+
+            __syncthreads();
+        }
+    }
+#ifdef NDEBUG
+#else
+    // ----------------------------------------------------------
+    // extra check that map[] is restored to identity permutation
+    // ----------------------------------------------------------
+    __syncthreads();
+    for(auto k = k_start; k < nn; k += k_inc)
+    {
+        assert(map[k] == k);
+    }
+    __syncthreads();
+#endif
+}
+
 // **********************************************************
 // GPU kernels that are used by many rocsolver functions
 // **********************************************************
@@ -96,8 +600,8 @@ enum copymat_direction
     copymat_from_buffer
 };
 
-/** A mask that is always true. Typically used to make the mask optional,
-    by acting as the default when no other mask is provided. **/
+/** A mask that is always true. Typically used to make the mask optional, by acting as the default when
+    no other mask is provided. **/
 struct no_mask
 {
     __device__ constexpr bool operator[](rocblas_int) const noexcept
@@ -106,7 +610,9 @@ struct no_mask
     }
 };
 
-/** An mask defined by an integer array (e.g., the info array) **/
+/** An mask defined by an integer array (e.g., the info array). By default, a non-zero value for the ith integer
+    indicates that the data for batch i should be copied over. Data will not be copied if the mask value is zero.
+    This behaviour is reversed if the negate transform is passed to the constructor. **/
 struct info_mask
 {
     enum mask_transform
@@ -268,15 +774,15 @@ ROCSOLVER_KERNEL void copy_mat(copymat_direction direction,
     An optional mask can be provided to limit the copy to selected matricies in the batch
     If uplo = rocblas_fill_upper, only the upper triangular part is copied
     If uplo = rocblas_fill_lower, only the lower triangular part is copied **/
-template <typename T1, typename T2, typename Mask = no_mask>
+template <typename T1, typename T2, typename U1, typename U2, typename Mask = no_mask>
 ROCSOLVER_KERNEL void copy_trans_mat(const rocblas_operation trans,
                                      const rocblas_int m,
                                      const rocblas_int n,
-                                     T1* A,
+                                     U1 A,
                                      const rocblas_int shiftA,
                                      const rocblas_int lda,
                                      const rocblas_stride strideA,
-                                     T2* B,
+                                     U2 B,
                                      const rocblas_int shiftB,
                                      const rocblas_int ldb,
                                      const rocblas_stride strideB,
@@ -335,39 +841,39 @@ ROCSOLVER_KERNEL void init_ident(const rocblas_int m,
     }
 }
 
-template <typename T, typename U>
-ROCSOLVER_KERNEL void reset_info(T* info, const rocblas_int n, U val)
+template <typename T, typename I, typename U>
+ROCSOLVER_KERNEL void reset_info(T* info, const I n, U val, I incr = 0)
 {
-    int idx = hipBlockIdx_x * hipBlockDim_x + hipThreadIdx_x;
+    I idx = hipBlockIdx_x * static_cast<I>(hipBlockDim_x) + hipThreadIdx_x;
 
     if(idx < n)
-        info[idx] = T(val);
+        info[idx] = T(val) + incr * idx;
 }
 
-template <typename T, typename S, typename U>
-ROCSOLVER_KERNEL void reset_batch_info(U info, const rocblas_stride stride, const rocblas_int n, S val)
+template <typename T, typename I, typename S, typename U>
+ROCSOLVER_KERNEL void reset_batch_info(U info, const rocblas_stride stride, const I n, S val)
 {
-    int idx = hipBlockIdx_x * hipBlockDim_x + hipThreadIdx_x;
-    int b = hipBlockIdx_y;
+    I idx = hipBlockIdx_x * static_cast<I>(hipBlockDim_x) + hipThreadIdx_x;
+    I b = hipBlockIdx_y;
 
     T* inf = load_ptr_batch<T>(info, b, 0, stride);
     if(idx < n)
         inf[idx] = T(val);
 }
 
-template <typename T>
-ROCSOLVER_KERNEL void get_array(T** out, T* in, rocblas_stride stride, rocblas_int batch)
+template <typename T, typename I>
+ROCSOLVER_KERNEL void get_array(T** out, T* in, rocblas_stride stride, I batch)
 {
-    int b = hipBlockIdx_x * hipBlockDim_x + hipThreadIdx_x;
+    I b = hipBlockIdx_x * static_cast<I>(hipBlockDim_x) + hipThreadIdx_x;
 
     if(b < batch)
         out[b] = in + b * stride;
 }
 
-template <typename T, typename U>
-ROCSOLVER_KERNEL void shift_array(T** out, U in, rocblas_int shift, rocblas_int batch)
+template <typename T, typename I, typename U>
+ROCSOLVER_KERNEL void shift_array(T** out, U in, rocblas_stride shift, I batch)
 {
-    int b = hipBlockIdx_x * hipBlockDim_x + hipThreadIdx_x;
+    I b = hipBlockIdx_x * static_cast<I>(hipBlockDim_x) + hipThreadIdx_x;
 
     if(b < batch)
         out[b] = in[b] + shift;
@@ -405,22 +911,23 @@ ROCSOLVER_KERNEL void restau(const rocblas_int k, T* ipiv, const rocblas_stride
 }
 
 template <typename T,
+          typename I,
           typename S,
           typename U,
           std::enable_if_t<!rocblas_is_complex<T> || rocblas_is_complex<S>, int> = 0>
 ROCSOLVER_KERNEL void set_diag(S* D,
-                               const rocblas_int shiftd,
+                               const rocblas_stride shiftd,
                                const rocblas_stride strided,
                                U A,
-                               const rocblas_int shifta,
-                               const rocblas_int lda,
+                               const rocblas_stride shifta,
+                               const I lda,
                                const rocblas_stride stridea,
-                               const rocblas_int n,
+                               const I n,
                                bool set_one)
 {
-    int b = hipBlockIdx_x;
-    int i = hipBlockIdx_y * hipBlockDim_y + hipThreadIdx_y;
-    int j = i + i * lda;
+    I b = hipBlockIdx_x;
+    I i = hipBlockIdx_y * hipBlockDim_y + hipThreadIdx_y;
+    I j = i + i * lda;
 
     S* d = load_ptr_batch<S>(D, b, shiftd, strided);
     T* a = load_ptr_batch<T>(A, b, shifta, stridea);
@@ -433,22 +940,23 @@ ROCSOLVER_KERNEL void set_diag(S* D,
 }
 
 template <typename T,
+          typename I,
           typename S,
           typename U,
           std::enable_if_t<rocblas_is_complex<T> && !rocblas_is_complex<S>, int> = 0>
 ROCSOLVER_KERNEL void set_diag(S* D,
-                               const rocblas_int shiftd,
+                               const rocblas_stride shiftd,
                                const rocblas_stride strided,
                                U A,
-                               const rocblas_int shifta,
-                               const rocblas_int lda,
+                               const rocblas_stride shifta,
+                               const I lda,
                                const rocblas_stride stridea,
-                               const rocblas_int n,
+                               const I n,
                                bool set_one)
 {
-    int b = hipBlockIdx_x;
-    int i = hipBlockIdx_y * hipBlockDim_y + hipThreadIdx_y;
-    int j = i + i * lda;
+    I b = hipBlockIdx_x;
+    I i = hipBlockIdx_y * hipBlockDim_y + hipThreadIdx_y;
+    I j = i + i * lda;
 
     S* d = load_ptr_batch<S>(D, b, shiftd, strided);
     T* a = load_ptr_batch<T>(A, b, shifta, stridea);
@@ -460,19 +968,19 @@ ROCSOLVER_KERNEL void set_diag(S* D,
     }
 }
 
-template <typename T, typename S, typename U>
+template <typename T, typename I, typename S, typename U>
 ROCSOLVER_KERNEL void restore_diag(S* D,
-                                   const rocblas_int shiftd,
+                                   const rocblas_stride shiftd,
                                    const rocblas_stride strided,
                                    U A,
-                                   const rocblas_int shifta,
-                                   const rocblas_int lda,
+                                   const rocblas_stride shifta,
+                                   const I lda,
                                    const rocblas_stride stridea,
-                                   const rocblas_int n)
+                                   const I n)
 {
-    int b = hipBlockIdx_x;
-    int i = hipBlockIdx_y * hipBlockDim_y + hipThreadIdx_y;
-    int j = i + i * lda;
+    I b = hipBlockIdx_x;
+    I i = hipBlockIdx_y * hipBlockDim_y + hipThreadIdx_y;
+    I j = i + i * lda;
 
     S* d = load_ptr_batch<S>(D, b, shiftd, strided);
     T* a = load_ptr_batch<T>(A, b, shifta, stridea);
@@ -750,3 +1258,47 @@ ROCSOLVER_KERNEL void check_singularity(const rocblas_int n,
     if(hipThreadIdx_y == 0)
         info[b] = _info;
 }
+
+/** SWAP swaps the values of vectors x and y of dimension n.
+    Launch this kernel with a desired number of threads organized in
+    NG groups in the x direction with NT threads in the x direction. **/
+template <typename S, typename T, typename I>
+ROCSOLVER_KERNEL void swap_kernel(I const n, T* const x, I const incx, T* const y, I const incy)
+{
+    if(n <= 0)
+        return;
+
+    I const i_start = hipThreadIdx_x + hipBlockIdx_x * hipBlockDim_x;
+    I const i_inc = hipBlockDim_x * hipGridDim_x;
+
+    if((incx == 1) && (incy == 1))
+    {
+        // ------------
+        // special case
+        // ------------
+        for(I i = i_start; i < n; i += i_inc)
+        {
+            auto const temp = y[i];
+            y[i] = x[i];
+            x[i] = temp;
+        }
+    }
+    else
+    {
+        // ---------------------------
+        // code for unequal increments
+        // ---------------------------
+
+        for(I i = i_start; i < n; i += i_inc)
+        {
+            auto const ix = 0 + i * static_cast<int64_t>(incx);
+            auto const iy = 0 + i * static_cast<int64_t>(incy);
+
+            auto const temp = y[iy];
+            y[iy] = x[ix];
+            x[ix] = temp;
+        }
+    }
+}
+
+ROCSOLVER_END_NAMESPACE
