@@ -4,8 +4,31 @@
  *     Univ. of Tennessee, Univ. of California Berkeley,
  *     Univ. of Colorado Denver and NAG Ltd..
  *     April 2012
- * Copyright (c) 2020-2022 Advanced Micro Devices, Inc.
- * ***********************************************************************/
+ * Copyright (C) 2020-2025 Advanced Micro Devices, Inc. All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
+ *
+ * 1. Redistributions of source code must retain the above copyright
+ *    notice, this list of conditions and the following disclaimer.
+ *
+ * 2. Redistributions in binary form must reproduce the above copyright
+ *    notice, this list of conditions and the following disclaimer in the
+ *    documentation and/or other materials provided with the distribution.
+ *
+ * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
+ * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+ * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+ * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
+ * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
+ * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
+ * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
+ * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
+ * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
+ * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
+ * SUCH DAMAGE.
+ * *************************************************************************/
 
 #pragma once
 
@@ -16,6 +39,9 @@
 #include "roclapack_geqrf.hpp"
 #include "roclapack_syevj_heevj.hpp"
 #include "rocsolver/rocsolver.h"
+#include "rocsolver_run_specialized_kernels.hpp"
+
+ROCSOLVER_BEGIN_NAMESPACE
 
 template <typename T, typename SS>
 ROCSOLVER_KERNEL void gesvdj_finalize(const rocblas_int n,
@@ -291,10 +317,9 @@ rocblas_status rocsolver_gesvdj_template(rocblas_handle handle,
         rocblas_int ldv_gemm = n;
         rocblas_int strideV_gemm = n * n;
 
-        rocblasCall_gemm<BATCHED, STRIDED, T>(
-            handle, rocblas_operation_conjugate_transpose, rocblas_operation_none, n, n, m, &minone,
-            A, shiftA, lda, strideA, A, shiftA, lda, strideA, &zero, V_gemm, 0, ldv_gemm,
-            strideV_gemm, batch_count, (T**)work6_workArr);
+        rocsolver_gemm(handle, rocblas_operation_conjugate_transpose, rocblas_operation_none, n, n,
+                       m, &minone, A, shiftA, lda, strideA, A, shiftA, lda, strideA, &zero, V_gemm,
+                       0, ldv_gemm, strideV_gemm, batch_count, (T**)work6_workArr);
 
         // apply eigenvalue decomposition to -A'A, obtaining V as eigenvectors
         rocsolver_syevj_heevj_template<false, STRIDED, T>(
@@ -308,10 +333,9 @@ rocblas_status rocsolver_gesvdj_template(rocblas_handle handle,
         rocblas_int ldu_gemm = (leftv ? ldu : m);
         rocblas_int strideU_gemm = (leftv ? strideU : m * n);
 
-        rocblasCall_gemm<BATCHED, STRIDED, T>(handle, rocblas_operation_none, rocblas_operation_none,
-                                              m, n, n, &one, A, shiftA, lda, strideA, V_gemm, 0,
-                                              ldv_gemm, strideV_gemm, &zero, U_gemm, 0, ldu_gemm,
-                                              strideU_gemm, batch_count, (T**)work6_workArr);
+        rocsolver_gemm(handle, rocblas_operation_none, rocblas_operation_none, m, n, n, &one, A,
+                       shiftA, lda, strideA, V_gemm, 0, ldv_gemm, strideV_gemm, &zero, U_gemm, 0,
+                       ldu_gemm, strideU_gemm, batch_count, (T**)work6_workArr);
 
         // apply QR factorization to AV, obtaining U = Q and S = R
         rocsolver_geqrf_template<false, STRIDED, T>(handle, m, n, U_gemm, 0, ldu_gemm, strideU_gemm,
@@ -332,7 +356,7 @@ rocblas_status rocsolver_gesvdj_template(rocblas_handle handle,
         if(rightv)
         {
             rocblas_int blocks_n = (n - 1) / BS2 + 1;
-            ROCSOLVER_LAUNCH_KERNEL(copy_trans_mat<T>, dim3(blocks_n, blocks_n, batch_count),
+            ROCSOLVER_LAUNCH_KERNEL((copy_trans_mat<T, T>), dim3(blocks_n, blocks_n, batch_count),
                                     dim3(BS2, BS2, 1), 0, stream,
                                     rocblas_operation_conjugate_transpose, n, n, V_gemm, 0,
                                     ldv_gemm, strideV_gemm, V, 0, ldv, strideV);
@@ -345,10 +369,9 @@ rocblas_status rocsolver_gesvdj_template(rocblas_handle handle,
         rocblas_int ldu_gemm = (leftv ? ldu : m);
         rocblas_int strideU_gemm = (leftv ? strideU : m * m);
 
-        rocblasCall_gemm<BATCHED, STRIDED, T>(
-            handle, rocblas_operation_none, rocblas_operation_conjugate_transpose, m, m, n, &minone,
-            A, shiftA, lda, strideA, A, shiftA, lda, strideA, &zero, U_gemm, 0, ldu_gemm,
-            strideU_gemm, batch_count, (T**)work6_workArr);
+        rocsolver_gemm(handle, rocblas_operation_none, rocblas_operation_conjugate_transpose, m, m,
+                       n, &minone, A, shiftA, lda, strideA, A, shiftA, lda, strideA, &zero, U_gemm,
+                       0, ldu_gemm, strideU_gemm, batch_count, (T**)work6_workArr);
 
         // apply eigenvalue decomposition to -AA', obtaining U as eigenvectors
         rocsolver_syevj_heevj_template<false, STRIDED, T>(
@@ -362,10 +385,9 @@ rocblas_status rocsolver_gesvdj_template(rocblas_handle handle,
         rocblas_int ldv_gemm = (rightv ? ldv : m);
         rocblas_int strideV_gemm = (rightv ? strideV : m * n);
 
-        rocblasCall_gemm<BATCHED, STRIDED, T>(
-            handle, rocblas_operation_conjugate_transpose, rocblas_operation_none, m, n, m, &one,
-            U_gemm, 0, ldu_gemm, strideU_gemm, A, shiftA, lda, strideA, &zero, V_gemm, 0, ldv_gemm,
-            strideV_gemm, batch_count, (T**)work6_workArr);
+        rocsolver_gemm(handle, rocblas_operation_conjugate_transpose, rocblas_operation_none, m, n,
+                       m, &one, U_gemm, 0, ldu_gemm, strideU_gemm, A, shiftA, lda, strideA, &zero,
+                       V_gemm, 0, ldv_gemm, strideV_gemm, batch_count, (T**)work6_workArr);
 
         // apply LQ factorization to U'A, obtaining S = L and V' = Q
         rocsolver_gelqf_template<false, STRIDED, T>(handle, m, n, V_gemm, 0, ldv_gemm, strideV_gemm,
@@ -386,3 +408,5 @@ rocblas_status rocsolver_gesvdj_template(rocblas_handle handle,
     rocblas_set_pointer_mode(handle, old_mode);
     return rocblas_status_success;
 }
+
+ROCSOLVER_END_NAMESPACE
