@@ -4,8 +4,31 @@
  *     Univ. of Tennessee, Univ. of California Berkeley,
  *     Univ. of Colorado Denver and NAG Ltd..
  *     December 2016
- * Copyright (c) 2021-2022 Advanced Micro Devices, Inc.
- * ***********************************************************************/
+ * Copyright (C) 2021-2024 Advanced Micro Devices, Inc. All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
+ *
+ * 1. Redistributions of source code must retain the above copyright
+ *    notice, this list of conditions and the following disclaimer.
+ *
+ * 2. Redistributions in binary form must reproduce the above copyright
+ *    notice, this list of conditions and the following disclaimer in the
+ *    documentation and/or other materials provided with the distribution.
+ *
+ * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
+ * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+ * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+ * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
+ * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
+ * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
+ * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
+ * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
+ * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
+ * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
+ * SUCH DAMAGE.
+ * *************************************************************************/
 
 #pragma once
 
@@ -14,6 +37,8 @@
 #include "rocblas.hpp"
 #include "rocsolver/rocsolver.h"
 
+ROCSOLVER_BEGIN_NAMESPACE
+
 template <typename T, typename U>
 ROCSOLVER_KERNEL void sygs2_set_diag1(const rocblas_int k,
                                       U AA,
@@ -133,7 +158,7 @@ void rocsolver_sygs2_hegs2_getMemorySize(const rocblas_eform itype,
     if(itype == rocblas_eform_ax)
     {
         // extra workspace (for calling TRSV)
-        *size_store_wcs = max(*size_store_wcs, sizeof(rocblas_int) * batch_count);
+        *size_store_wcs = std::max(*size_store_wcs, sizeof(rocblas_int) * batch_count);
         *size_work = 0;
     }
     else
@@ -214,7 +239,7 @@ rocblas_status rocsolver_sygs2_hegs2_template(rocblas_handle handle,
     rocblas_int blocks_batch = (batch_count - 1) / BS1 + 1;
     rocblas_int waves_batch = (batch_count - 1) / warpSize + 1;
     dim3 blocks(blocks_batch, 1, 1);
-    dim3 threads(min(BS1, warpSize * waves_batch), 1, 1);
+    dim3 threads(std::min(BS1, warpSize * waves_batch), 1, 1);
 
     if(itype == rocblas_eform_ax)
     {
@@ -263,11 +288,11 @@ rocblas_status rocsolver_sygs2_hegs2_template(rocblas_handle handle,
                                                     shiftB + idx2D(k, k + 1, ldb), ldb, strideB,
                                                     batch_count);
 
-                    rocblasCall_trsv<BATCHED, T>(handle, uplo, rocblas_operation_conjugate_transpose,
-                                                 rocblas_diagonal_non_unit, n - k - 1, B,
-                                                 shiftB + idx2D(k + 1, k + 1, ldb), ldb, strideB, A,
-                                                 shiftA + idx2D(k, k + 1, lda), lda, strideA,
-                                                 batch_count, (rocblas_int*)store_wcs, workArr);
+                    rocblasCall_trsv(handle, uplo, rocblas_operation_conjugate_transpose,
+                                     rocblas_diagonal_non_unit, n - k - 1, B,
+                                     shiftB + idx2D(k + 1, k + 1, ldb), ldb, strideB, A,
+                                     shiftA + idx2D(k, k + 1, lda), lda, strideA, batch_count,
+                                     (rocblas_int*)store_wcs, workArr);
 
                     if(COMPLEX)
                         rocsolver_lacgv_template<T>(handle, n - k - 1, A,
@@ -304,11 +329,10 @@ rocblas_status rocsolver_sygs2_hegs2_template(rocblas_handle handle,
                                         shiftB + idx2D(k + 1, k, ldb), 1, strideB, A,
                                         shiftA + idx2D(k + 1, k, lda), 1, strideA, batch_count);
 
-                    rocblasCall_trsv<BATCHED, T>(handle, uplo, rocblas_operation_none,
-                                                 rocblas_diagonal_non_unit, n - k - 1, B,
-                                                 shiftB + idx2D(k + 1, k + 1, ldb), ldb, strideB, A,
-                                                 shiftA + idx2D(k + 1, k, lda), 1, strideA,
-                                                 batch_count, (rocblas_int*)store_wcs, workArr);
+                    rocblasCall_trsv(handle, uplo, rocblas_operation_none, rocblas_diagonal_non_unit,
+                                     n - k - 1, B, shiftB + idx2D(k + 1, k + 1, ldb), ldb, strideB,
+                                     A, shiftA + idx2D(k + 1, k, lda), 1, strideA, batch_count,
+                                     (rocblas_int*)store_wcs, workArr);
                 }
             }
         }
@@ -408,3 +432,5 @@ rocblas_status rocsolver_sygs2_hegs2_template(rocblas_handle handle,
     rocblas_set_pointer_mode(handle, old_mode);
     return rocblas_status_success;
 }
+
+ROCSOLVER_END_NAMESPACE
